
import pygame
import random
import sys
from os import path

WIDTH = 480
HEIGHT = 600
FPS = 60
POWERUP_TIME = 5000

# define colors
WHITE = (255,255,255)
BLACK = (0,0,0)
RED = (255,0,0)
GREEN = (0,255,0)
BLUE = (0,0,255)

# 绘制文本
def draw_text(surf, text, size, x, y):
    font = pygame.font.Font(path.join('star war/font/simkai.ttf'), size)
    text_surface = font.render(text, True, WHITE)
    text_rect = text_surface.get_rect()
    text_rect.midtop = (x, y)
    surf.blit(text_surface, text_rect)

def newmob():
    m = Mob()
    all_sprites.add(m)
    mobs.add(m)

# 生命显示
def draw_lives(surf,lives):
    for i in range(lives):
        img_rect = player_mini_img.get_rect()
        img_rect.x = (WIDTH-100)+30*i
        img_rect.y = 5
        surf.blit(player_mini_img,img_rect)
# 显示开始页面
def show_go_screen():
    screen.blit(background,background_rect)
    draw_text(screen,'star wars',64,WIDTH/2,HEIGHT/4)
    draw_text(screen,'left right to move space to shoot',22,WIDTH/2,HEIGHT/2)
    draw_text(screen,'press space to start',18,WIDTH/2,HEIGHT*3/4)
    pygame.display.flip()
    while True:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.KEYUP:
                return



class Player(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.image = pygame.transform.scale(player_img,(50,38))
        self.image.set_colorkey(BLACK)
        self.rect = self.image.get_rect()
        self.rect.centerx = WIDTH / 2
        self.rect.bottom = HEIGHT - 10
        self.speedx = 0

        # # 盾牌
        self.shield = 100

        self.shoot_delay = 250
        # 子弹的起始时间
        self.last_shot = 0
        self.lives = 3

        self.hidden = False
        self.hide_timer=0
        self.power = 1
    def update(self):

        # 判断子弹消失时间
        if self.power >= 2 and pygame.time.get_ticks() - self.power_time > 10000:
            self.power = 1
            self.power_time = pygame.time.get_ticks()
        self.speedx = 0
        # 获得所有键盘按钮的状态
        keystate = pygame.key.get_pressed()
        if keystate[pygame.K_LEFT]:
            self.speedx = -5
        if keystate[pygame.K_RIGHT]:
            self.speedx = 5
        if keystate[pygame.K_SPACE]:
            self.shoot()
        self.rect.x += self.speedx
        # 右侧大于边界
        if self.rect.right > WIDTH:
            self.rect.right = WIDTH
        if self.rect.left < 0:
            self.rect.left = 0

        if self.hidden and pygame.time.get_ticks() - self.hide_timer > 1000:
            self.hidden = False
            self.rect.centerx = WIDTH/2
            self.rect.bottom = HEIGHT-10
    # 射击
    def shoot(self):
        now = pygame.time.get_ticks()
    #                                     # 射击延迟250
        if now - self.last_shot > self.shoot_delay:
            self.last_shot = now
            if self.power == 1 :
                bullet3 = Bullet(self.rect.centerx, self.rect.top)
                all_sprites.add(bullet3)
                # 为了石头和子弹碰撞检测
                bullets.add(bullet3)
            elif self.power == 2 :
                bullet1 = Bullet(self.rect.left, self.rect.centery)
                bullet2 = Bullet(self.rect.right, self.rect.centery)
                all_sprites.add(bullet1)
                all_sprites.add(bullet2)
                bullets.add(bullet1)
                bullets.add(bullet2)
            elif self.power >= 3:
               # 子弹函数
                bullet1 = Bullet(self.rect.left,self.rect.centery)
                bullet2 = Bullet(self.rect.right,self.rect.centery)
                bullet3 = Bullet(self.rect.centerx, self.rect.top)
                all_sprites.add(bullet1)
                all_sprites.add(bullet2)
                all_sprites.add(bullet3)
                # 为了发生碰撞检测
                bullets.add(bullet1)
                bullets.add(bullet2)
                bullets.add(bullet3)
            shoot_sound.play()

    def hide(self):
        # hide the player temporarily
        self.hidden = True
        self.hide_timer = pygame.time.get_ticks()
        self.rect.center = (WIDTH/2,HEIGHT+200)

    def powerup(self):
            self.power += 1
            self.power_time = pygame.time.get_ticks()

class Mob(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        # 随机选择石头图片 ，列表里有6个
        self.image_orin = random.choice(meteor_images)
        self.image = self.image_orin.copy()
        self.rect = self.image.get_rect()

        # 在屏幕上面准备                   # 400-图片的宽
        self.rect.x = random.randrange(WIDTH-self.rect.width)
        self.rect.y = random.randrange(-100,-40)
        self.speedy = random.randrange(1,8)
        self.speedx = random.randrange(-3,3)

        self.radius = int(self.rect.width / 2)
        # 坠落
        self.rot = 0
        self.rot_speed = random.randrange(-8,8)
        self.last_update = 0
    # 旋转石头
    def rotate(self):
        now = pygame.time.get_ticks()
        if now - self.last_update > 50:
            self.last_update = now
            self.rot = (self.rot + self.rot_speed) % 360
            # pygame.transform.rotate(surface, angle)
            # 返回一个旋转后的surface对象，默认是按照逆时针进行旋转的，当angle小于0时则代表的顺时针进行旋转
                            # 变换转变            石头图片       旋转角度
            new_image = pygame.transform.rotate(self.image_orin,self.rot)
            # 拿到图片的中心圆点
            old_center = self.rect.center
            self.image = new_image
            self.rect = self.image.get_rect()
            self.rect.center = old_center

    def update(self):
        self.rotate()
        self.rect.x += self.speedx
        self.rect.y += self.speedy
        if self.rect.top > HEIGHT + 10 or self.rect.left < -25 or self.rect.right > WIDTH + 20:
            # 落下之后重新设置石头的位置和y坐标的速度
            self.rect.x = random.randrange(WIDTH-self.rect.width)
            self.rect.y = random.randrange(-100,-40)
            # x坐标方向移动只设置一次，y坐标移动速度会变
            self.speedy = random.randrange(1,8)

# 子弹类
class Bullet(pygame.sprite.Sprite):
    def __init__(self,x,y):
        pygame.sprite.Sprite.__init__(self)
        self.image = pygame.image.load('star war/img/laserRed16.png').convert()
        self.image.set_colorkey(BLACK)
        self.rect = self.image.get_rect()
        self.rect.bottom = y
        self.rect.centerx = x
        self.speedy = -10

    def update(self):
        self.rect.y += self.speedy
        if self.rect.bottom < 0:
            self.kill()

# 爆炸
class Explosion(pygame.sprite.Sprite):
                # 传入石头的中心点
    def __init__(self,center,size):
        pygame.sprite.Sprite.__init__(self)
        self.explosion_anim = {}
        self.explosion_anim['lg'] = []

        self.explosion_anim['player'] = []
        # 爆炸的圈
        for i in range(9):
            filename= 'star war/img/regularExplosion0%s.png' % i
            img = pygame.image.load(filename).convert_alpha()
            img_lg = pygame.transform.scale(img, (75, 75))
            self.explosion_anim['lg'].append(img_lg)
            filename = 'star war/img/sonicExplosion0%s.png' % i
            img = pygame.image.load(filename).convert_alpha()
            self.explosion_anim['player'].append(img)

        self.size = size
        # 字典通过键拿出爆炸图片的"lg" [75,75]尺寸，拿出第一张
        self.image = self.explosion_anim[self.size][0]
        self.rect = self.image.get_rect()
        # 放在石头的中心点处
        self.rect.center = center
        # 画面
        self.frame = 0
        self.last_update = pygame.time.get_ticks()
        self.frame_rate = 75

    def update(self):
        now = pygame.time.get_ticks()
        if now - self.last_update > self.frame_rate:
            self.last_update = now
            self.frame += 1
                                # 爆炸字典里拿出来的值：是列表 9张图片
            if self.frame == len(self.explosion_anim[self.size]):
                self.kill()
            else:
                # 从第二张图片开始，爆炸的中心点就是石头的中心点
                center = self.rect.center
                self.image = self.explosion_anim[self.size][self.frame]
                self.rect = self.image.get_rect()
                self.rect.center = center

# 能量包括盾和闪电  随机拿出一张图片
class Pow(pygame.sprite.Sprite):
    def __init__(self,center):
        pygame.sprite.Sprite.__init__(self)
        self.powerup_images = []
        # 闪电
        self.type = random.choice(['shield', 'lightning'])

        if self.type == 'shield':
            self.img_shield = pygame.image.load('star war/img/shield_gold.png').convert_alpha()
            self.powerup_images.append(self.img_shield)
        else:
            self.img_bolt = pygame.image.load('star war/img/bolt_gold.png').convert_alpha()
            self.powerup_images.append(self.img_bolt)
        self.image=self.powerup_images[0]
        self.rect = self.image.get_rect()
        self.rect.center = center


    def update(self):
        self.speedy = 5
        self.rect.y += self.speedy
        # kill if it moves off the top of the screen
        if self.rect.bottom > HEIGHT:
            self.kill()

pygame.init()
pygame.mixer.init()
screen = pygame.display.set_mode((WIDTH,HEIGHT))
pygame.display.set_caption("starwars")
clock = pygame.time.Clock()
                    # 在系统上找到特定的字体
font_name = pygame.font.match_font('arial')
background = pygame.image.load('star war/img/background.png').convert()
background_rect = background.get_rect()
player_img = pygame.image.load('star war/img/playerShip1_orange.png').convert()
                                # 图片缩放
player_mini_img = pygame.transform.scale(player_img,(25,19))
                # 设置透明色
player_mini_img.set_colorkey(BLACK)

meteor_images = []
# 添加石头图片
meteor=["star war/img/meteorBrown_med1.png","star war/img/meteorBrown_med3.png"
    ,"star war/img/meteorBrown_small1.png","star war/img/meteorBrown_small2.png"
    ,"star war/img/meteorBrown_tiny1.png","star war/img/meteorBrown_tiny2.png"]

for img in meteor:
    meteor_images.append(pygame.image.load(img).convert_alpha())

# Load all game sounds
shoot_sound = pygame.mixer.Sound('star war/snd/laser_shoot.wav')
# 盾的声音
shield_sound = pygame.mixer.Sound('star war/snd/shield.wav')

power_sound = pygame.mixer.Sound('star war/snd/powerup.wav')


# 爆炸的声音
expl_sounds = []
for snd in ['star war/snd/explosion9.wav','star war/snd/explosion.wav','star war/snd/explosion10.wav']:
    expl_sounds.append(pygame.mixer.Sound(path.join(snd)))

# 石头碰撞飞机的声音
player_dir_sound = pygame.mixer.Sound(path.join('star war/snd/rumble1.ogg'))

pygame.mixer.music.load(path.join('star war/snd/tgfcoder-FrozenJam-SeamlessLoop.ogg'))
                    # 设置音量
pygame.mixer.music.set_volume(0.4)
                # 开始播放音乐流，重复开始播放
pygame.mixer.music.play(loops=-1)

game_over = True
while True:
    if game_over:
        # 按键抬起结束初始界面
        show_go_screen()
        game_over = False
        all_sprites = pygame.sprite.Group()
        mobs = pygame.sprite.Group()
        # 子弹组
        bullets = pygame.sprite.Group()
        powerups = pygame.sprite.Group()
        player = Player()
        all_sprites.add(player)

        for i in range(8):
            newmob()
        score = 0

    for event in pygame.event.get():
        # check for closing window
        if event.type == pygame.QUIT:
            pygame.quit()
            sys.exit()


    # check to see if a bullet hit a mobs

    # 石头和子弹的碰撞  前面为键 后面的为值
    hits = pygame.sprite.groupcollide(mobs,bullets,True,True)
    # random.choice(expl_sounds).play()
    for hit in hits:
        score += 50 - hit.radius
        # 三种爆炸声音中选择一种播放
        random.choice(expl_sounds).play()
                # 爆炸类    传入石头的中心点，
        expl = Explosion(hit.rect.center,'lg')
        all_sprites.add(expl)
        newmob()
        if random.random() > 0.9:
        #     # 实例化一个能量图片
            power = Pow(hit.rect.center)
            all_sprites.add(power)
            powerups.add(power)

    # 石头碰撞飞机
    # # check to see if a mob hit the player
    hits = pygame.sprite.spritecollide(player,mobs,True)
    if hits:
            player_dir_sound.play()
            death_explosion = Explosion(player.rect.center,'player')
            all_sprites.add(death_explosion)
            # 隐藏飞机
            player.hide()
            player.lives -= 1
            newmob()

    # 能量碰撞飞机
    # # check to see if player hit a powerup
    hits = pygame.sprite.spritecollide(player,powerups,True)
    for hit in hits:
        # 子弹个数闪电
        if hit.type == 'lightning':
            power_sound.play()
            # 调用能量
            player.powerup()

        # 盾牌加命
        if hit.type == 'shield':
            shield_sound.play()
            player.lives += 1
            # 超出范围
            if player.lives > 3:
                player.lives = 3

    screen.blit(background,background_rect)
    # 对精灵组中的每一个精灵依次调用surface.blit()，依次将精灵组中的精灵绘制在surface上
    clock.tick(FPS)
    all_sprites.update()
    all_sprites.draw(screen)
    # 写分数
    draw_text(screen, str(score), 18, WIDTH/2, 10)
    draw_lives(screen,player.lives)
    
    import pygame
    import random
    import sys
    from os import path

    WIDTH = 480
    HEIGHT = 600
    FPS = 60
    POWERUP_TIME = 5000

    # define colors
    WHITE = (255, 255, 255)
    BLACK = (0, 0, 0)
    RED = (255, 0, 0)
    GREEN = (0, 255, 0)
    BLUE = (0, 0, 255)


    # 绘制文本
    def draw_text(surf, text, size, x, y):
        font = pygame.font.Font(path.join('star war/font/simkai.ttf'), size)
        text_surface = font.render(text, True, WHITE)
        text_rect = text_surface.get_rect()
        text_rect.midtop = (x, y)
        surf.blit(text_surface, text_rect)


    def newmob():
        m = Mob()
        all_sprites.add(m)
        mobs.add(m)


    # 生命显示
    def draw_lives(surf, lives):
        for i in range(lives):
            img_rect = player_mini_img.get_rect()
            img_rect.x = (WIDTH - 100) + 30 * i
            img_rect.y = 5
            surf.blit(player_mini_img, img_rect)


    # 显示开始页面
    def show_go_screen():
        screen.blit(background, background_rect)
        draw_text(screen, 'star wars', 64, WIDTH / 2, HEIGHT / 4)
        draw_text(screen, 'left right to move space to shoot', 22, WIDTH / 2, HEIGHT / 2)
        draw_text(screen, 'press space to start', 18, WIDTH / 2, HEIGHT * 3 / 4)
        pygame.display.flip()
        while True:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
                if event.type == pygame.KEYUP:
                    return


    class Player(pygame.sprite.Sprite):
        def __init__(self):
            pygame.sprite.Sprite.__init__(self)
            self.image = pygame.transform.scale(player_img, (50, 38))
            self.image.set_colorkey(BLACK)
            self.rect = self.image.get_rect()
            self.rect.centerx = WIDTH / 2
            self.rect.bottom = HEIGHT - 10
            self.speedx = 0

            # # 盾牌
            self.shield = 100

            self.shoot_delay = 250
            # 子弹的起始时间
            self.last_shot = 0
            self.lives = 3

            self.hidden = False
            self.hide_timer = 0
            self.power = 1

        def update(self):

            # 判断子弹消失时间
            if self.power >= 2 and pygame.time.get_ticks() - self.power_time > 10000:
                self.power = 1
                self.power_time = pygame.time.get_ticks()
            self.speedx = 0
            # 获得所有键盘按钮的状态
            keystate = pygame.key.get_pressed()
            if keystate[pygame.K_LEFT]:
                self.speedx = -5
            if keystate[pygame.K_RIGHT]:
                self.speedx = 5
            if keystate[pygame.K_SPACE]:
                self.shoot()
            self.rect.x += self.speedx
            # 右侧大于边界
            if self.rect.right > WIDTH:
                self.rect.right = WIDTH
            if self.rect.left < 0:
                self.rect.left = 0

            if self.hidden and pygame.time.get_ticks() - self.hide_timer > 1000:
                self.hidden = False
                self.rect.centerx = WIDTH / 2
                self.rect.bottom = HEIGHT - 10

        # 射击
        def shoot(self):
            now = pygame.time.get_ticks()
            #                                     # 射击延迟250
            if now - self.last_shot > self.shoot_delay:
                self.last_shot = now
                if self.power == 1:
                    bullet3 = Bullet(self.rect.centerx, self.rect.top)
                    all_sprites.add(bullet3)
                    # 为了石头和子弹碰撞检测
                    bullets.add(bullet3)
                elif self.power == 2:
                    bullet1 = Bullet(self.rect.left, self.rect.centery)
                    bullet2 = Bullet(self.rect.right, self.rect.centery)
                    all_sprites.add(bullet1)
                    all_sprites.add(bullet2)
                    bullets.add(bullet1)
                    bullets.add(bullet2)
                elif self.power >= 3:
                    # 子弹函数
                    bullet1 = Bullet(self.rect.left, self.rect.centery)
                    bullet2 = Bullet(self.rect.right, self.rect.centery)
                    bullet3 = Bullet(self.rect.centerx, self.rect.top)
                    all_sprites.add(bullet1)
                    all_sprites.add(bullet2)
                    all_sprites.add(bullet3)
                    # 为了发生碰撞检测
                    bullets.add(bullet1)
                    bullets.add(bullet2)
                    bullets.add(bullet3)
                shoot_sound.play()

        def hide(self):
            # hide the player temporarily
            self.hidden = True
            self.hide_timer = pygame.time.get_ticks()
            self.rect.center = (WIDTH / 2, HEIGHT + 200)

        def powerup(self):
            self.power += 1
            self.power_time = pygame.time.get_ticks()


    class Mob(pygame.sprite.Sprite):
        def __init__(self):
            pygame.sprite.Sprite.__init__(self)
            # 随机选择石头图片 ，列表里有6个
            self.image_orin = random.choice(meteor_images)
            self.image = self.image_orin.copy()
            self.rect = self.image.get_rect()

            # 在屏幕上面准备                   # 400-图片的宽
            self.rect.x = random.randrange(WIDTH - self.rect.width)
            self.rect.y = random.randrange(-100, -40)
            self.speedy = random.randrange(1, 8)
            self.speedx = random.randrange(-3, 3)

            self.radius = int(self.rect.width / 2)
            # 坠落
            self.rot = 0
            self.rot_speed = random.randrange(-8, 8)
            self.last_update = 0

        # 旋转石头
        def rotate(self):
            now = pygame.time.get_ticks()
            if now - self.last_update > 50:
                self.last_update = now
                self.rot = (self.rot + self.rot_speed) % 360
                # pygame.transform.rotate(surface, angle)
                # 返回一个旋转后的surface对象，默认是按照逆时针进行旋转的，当angle小于0时则代表的顺时针进行旋转
                # 变换转变            石头图片       旋转角度
                new_image = pygame.transform.rotate(self.image_orin, self.rot)
                # 拿到图片的中心圆点
                old_center = self.rect.center
                self.image = new_image
                self.rect = self.image.get_rect()
                self.rect.center = old_center

        def update(self):
            self.rotate()
            self.rect.x += self.speedx
            self.rect.y += self.speedy
            if self.rect.top > HEIGHT + 10 or self.rect.left < -25 or self.rect.right > WIDTH + 20:
                # 落下之后重新设置石头的位置和y坐标的速度
                self.rect.x = random.randrange(WIDTH - self.rect.width)
                self.rect.y = random.randrange(-100, -40)
                # x坐标方向移动只设置一次，y坐标移动速度会变
                self.speedy = random.randrange(1, 8)


    # 子弹类
    class Bullet(pygame.sprite.Sprite):
        def __init__(self, x, y):
            pygame.sprite.Sprite.__init__(self)
            self.image = pygame.image.load('star war/img/laserRed16.png').convert()
            self.image.set_colorkey(BLACK)
            self.rect = self.image.get_rect()
            self.rect.bottom = y
            self.rect.centerx = x
            self.speedy = -10

        def update(self):
            self.rect.y += self.speedy
            if self.rect.bottom < 0:
                self.kill()


    # 爆炸
    class Explosion(pygame.sprite.Sprite):
        # 传入石头的中心点
        def __init__(self, center, size):
            pygame.sprite.Sprite.__init__(self)
            self.explosion_anim = {}
            self.explosion_anim['lg'] = []

            self.explosion_anim['player'] = []
            # 爆炸的圈
            for i in range(9):
                filename = 'star war/img/regularExplosion0%s.png' % i
                img = pygame.image.load(filename).convert_alpha()
                img_lg = pygame.transform.scale(img, (75, 75))
                self.explosion_anim['lg'].append(img_lg)
                filename = 'star war/img/sonicExplosion0%s.png' % i
                img = pygame.image.load(filename).convert_alpha()
                self.explosion_anim['player'].append(img)

            self.size = size
            # 字典通过键拿出爆炸图片的"lg" [75,75]尺寸，拿出第一张
            self.image = self.explosion_anim[self.size][0]
            self.rect = self.image.get_rect()
            # 放在石头的中心点处
            self.rect.center = center
            # 画面
            self.frame = 0
            self.last_update = pygame.time.get_ticks()
            self.frame_rate = 75

        def update(self):
            now = pygame.time.get_ticks()
            if now - self.last_update > self.frame_rate:
                self.last_update = now
                self.frame += 1
                # 爆炸字典里拿出来的值：是列表 9张图片
                if self.frame == len(self.explosion_anim[self.size]):
                    self.kill()
                else:
                    # 从第二张图片开始，爆炸的中心点就是石头的中心点
                    center = self.rect.center
                    self.image = self.explosion_anim[self.size][self.frame]
                    self.rect = self.image.get_rect()
                    self.rect.center = center


    # 能量包括盾和闪电  随机拿出一张图片
    class Pow(pygame.sprite.Sprite):
        def __init__(self, center):
            pygame.sprite.Sprite.__init__(self)
            self.powerup_images = []
            # 闪电
            self.type = random.choice(['shield', 'lightning'])

            if self.type == 'shield':
                self.img_shield = pygame.image.load('star war/img/shield_gold.png').convert_alpha()
                self.powerup_images.append(self.img_shield)
            else:
                self.img_bolt = pygame.image.load('star war/img/bolt_gold.png').convert_alpha()
                self.powerup_images.append(self.img_bolt)
            self.image = self.powerup_images[0]
            self.rect = self.image.get_rect()
            self.rect.center = center

        def update(self):
            self.speedy = 5
            self.rect.y += self.speedy
            # kill if it moves off the top of the screen
            if self.rect.bottom > HEIGHT:
                self.kill()


    pygame.init()
    pygame.mixer.init()
    screen = pygame.display.set_mode((WIDTH, HEIGHT))
    pygame.display.set_caption("starwars")
    clock = pygame.time.Clock()
    # 在系统上找到特定的字体
    font_name = pygame.font.match_font('arial')
    background = pygame.image.load('star war/img/background.png').convert()
    background_rect = background.get_rect()
    player_img = pygame.image.load('star war/img/playerShip1_orange.png').convert()
    # 图片缩放
    player_mini_img = pygame.transform.scale(player_img, (25, 19))
    # 设置透明色
    player_mini_img.set_colorkey(BLACK)

    meteor_images = []
    # 添加石头图片
    meteor = ["star war/img/meteorBrown_med1.png", "star war/img/meteorBrown_med3.png"
        , "star war/img/meteorBrown_small1.png", "star war/img/meteorBrown_small2.png"
        , "star war/img/meteorBrown_tiny1.png", "star war/img/meteorBrown_tiny2.png"]

    for img in meteor:
        meteor_images.append(pygame.image.load(img).convert_alpha())

    # Load all game sounds
    shoot_sound = pygame.mixer.Sound('star war/snd/laser_shoot.wav')
    # 盾的声音
    shield_sound = pygame.mixer.Sound('star war/snd/shield.wav')

    power_sound = pygame.mixer.Sound('star war/snd/powerup.wav')

    # 爆炸的声音
    expl_sounds = []
    for snd in ['star war/snd/explosion9.wav', 'star war/snd/explosion.wav', 'star war/snd/explosion10.wav']:
        expl_sounds.append(pygame.mixer.Sound(path.join(snd)))

    # 石头碰撞飞机的声音
    player_dir_sound = pygame.mixer.Sound(path.join('star war/snd/rumble1.ogg'))

    pygame.mixer.music.load(path.join('star war/snd/tgfcoder-FrozenJam-SeamlessLoop.ogg'))
    # 设置音量
    pygame.mixer.music.set_volume(0.4)
    # 开始播放音乐流，重复开始播放
    pygame.mixer.music.play(loops=-1)

    game_over = True
    while True:
        if game_over:
            # 按键抬起结束初始界面
            show_go_screen()
            game_over = False
            all_sprites = pygame.sprite.Group()
            mobs = pygame.sprite.Group()
            # 子弹组
            bullets = pygame.sprite.Group()
            powerups = pygame.sprite.Group()
            player = Player()
            all_sprites.add(player)

            for i in range(8):
                newmob()
            score = 0

        for event in pygame.event.get():
            # check for closing window
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()

        # check to see if a bullet hit a mobs

        # 石头和子弹的碰撞  前面为键 后面的为值
        hits = pygame.sprite.groupcollide(mobs, bullets, True, True)
        # random.choice(expl_sounds).play()
        for hit in hits:
            score += 50 - hit.radius
            # 三种爆炸声音中选择一种播放
            random.choice(expl_sounds).play()
            # 爆炸类    传入石头的中心点，
            expl = Explosion(hit.rect.center, 'lg')
            all_sprites.add(expl)
            newmob()
            if random.random() > 0.9:
                #     # 实例化一个能量图片
                power = Pow(hit.rect.center)
                all_sprites.add(power)
                powerups.add(power)

        # 石头碰撞飞机
        # # check to see if a mob hit the player
        hits = pygame.sprite.spritecollide(player, mobs, True)
        if hits:
            player_dir_sound.play()
            death_explosion = Explosion(player.rect.center, 'player')
            all_sprites.add(death_explosion)
            # 隐藏飞机
            player.hide()
            player.lives -= 1
            newmob()

        # 能量碰撞飞机
        # # check to see if player hit a powerup
        hits = pygame.sprite.spritecollide(player, powerups, True)
        for hit in hits:
            # 子弹个数闪电
            if hit.type == 'lightning':
                power_sound.play()
                # 调用能量
                player.powerup()

            # 盾牌加命
            if hit.type == 'shield':
                shield_sound.play()
                player.lives += 1
                # 超出范围
                if player.lives > 3:
                    player.lives = 3

        screen.blit(background, background_rect)
        # 对精灵组中的每一个精灵依次调用surface.blit()，依次将精灵组中的精灵绘制在surface上
        clock.tick(FPS)
        all_sprites.update()
        all_sprites.draw(screen)
        # 写分数
        draw_text(screen, str(score), 18, WIDTH / 2, 10)
        draw_lives(screen, player.lives)

        #                             没有组包含爆炸图片，
        if player.lives == 0 and not death_explosion.alive():
            game_over = True

        pygame.display.flip()

    #                             没有组包含爆炸图片，
    if player.lives == 0 and not death_explosion.alive():
        game_over = True

    pygame.display.flip()





